//---------------------------------------------------------------------------------------
// Визуализация данных на трехзначном 7-сегментном индикаторе
// leddd05 v. 2016.11.01.10.25
// OldBean


//---------------------------------------------------------------------------------------
//#define COMMON_CATHODE // Если индикатор с общим анодом - необходимо закомментировать эту строку
//---------------------------------------------------------------------------------------
#ifdef COMMON_CATHODE // Для индикаторов с общим катодом
#define L_HIGH 1
#define L_LOW  0
#else                 // Для индикаторов с общим анодом
#define L_HIGH 0
#define L_LOW  1
#endif

#define BUTTON 16     // Кнопка управления
#define DEBUG 1       // признак отладки - пишем сообщения в serial

int button_press_time = 0;
unsigned long time_press = 0;
//byte button_press = 0;
int press_time = 0;
unsigned long t = 0; // таймер показа номера датчика
int t_delay = 2000; // время показа номера датчика
//---------------------------------------------------------------------------------------
// Пины, к которым подключены сегменты индикатора аноды (или катоды для индикаторов с общими анодами)
static uint8_t segPins[8] = { 11, 15, 6, 8, 9, 12, 5, 7};
//                           {12, 16, 5, 7, 8, 13, 4};



static float temps[4] = {0.745, 8.2, 35.9, 146.2};      // массив температур
byte current_sensor = 0;                                // текущий датчик


// Пины, к которым подключены общие катоды (или аноды, для индикаторов с общими анодами)
static uint8_t digPins[4] = {4, 14, 13, 10};
static uint8_t digit[22] = {0b00111111, // Маски сегментов цифр 0, 1, 2 и т.д.
                            0b00000110, //1  старший бит отвечает за точку
                            0b01011011, //2
                            0b01001111, //3
                            0b01100110, //4
                            0b01101101, //5
                            0b01111101, //6
                            0b00000111, //7
                            0b01111111, //8
                            0b01101111, //9
                            0b00000000, // "Цифра" 10 это пробел
                            
                            0b10111111, // Маски сегментов цифр 0, 1, 2 и т.д. с точками //11
                            0b10000110, //1  старший бит отвечает за точку 12
                            0b11011011, //2 13
                            0b11001111, //3 14
                            0b11100110, //4 15
                            0b11101101, //5 16
                            0b11111101, //6 17 
                            0b10000111, //7 18
                            0b11111111, //8 19 
                            0b11101111, //9 20
                            0b10000000}; // 21 - точка без цифры
static int d[4]; // Тысячи, сотни, десятки и единицы показываемые на разрядах 7-сегментного индикатора
static uint8_t mask = 0x01; // Маска для анализа цифр
static int seg = 0; // Номер сегмента, обрабатываемого (подствечиваемого, если нужно) в данный момент
//static int prSeg = 6; // Предыдущий сегмент
static unsigned long dt = 2000; // Длительность (в мкс) подсветки каждой группы сегментов
static volatile unsigned long stt = 0; // Момент времени (в мкс) когда заканчивается подсветка текущей группы сегментов
static volatile uint8_t nextSegFlag = 1; // Флаг для перехода к подсветке следующей группы сегментов

static volatile float rms = 0; // Среднеквадратичное значение измеряемого напряжения
static uint8_t updateFlag = 1; // 1 - данные (цифры), выводимые на индикатор, нужно обновить, 2 - выводим номер датчика

// Рабочие переменные для отладки
static volatile unsigned long stg = 0;
static volatile int cntr = 0;
//---------------------------------------------------------------------------------------
void setup() {
  for(int i = 0; i < 8; i++) { // Инициализация пинов, к которым подключены аноды индикатора (сегменты)
    pinMode(segPins[i], OUTPUT);
    digitalWrite(segPins[i], L_LOW);
  }
  for(int i = 0; i < 4; i++) { // Инициализация пинов, к которым подключены катоды индикатора (цифры)
    pinMode(digPins[i], OUTPUT);
    digitalWrite(digPins[i], L_HIGH);
  }
  unsigned long st = millis() + 3000;
  d[0] = 8; d[1] = 8; d[2] = 8; d[3] = 8;// Для демонстрации работоспособности индикатора покажем все сегменты разом
  while(millis() < st) {
      ShowSegments();
  }
  d[0] = 10; d[1] = 10; d[2] = 10; d[3] = 10; // Очистим индикатор
  ShowSegments();

  stg = micros() + dt;
  rms = temps[current_sensor];
  // Настраиваем пин кнопки
  pinMode(BUTTON, INPUT);
  digitalWrite(BUTTON, HIGH);
  Serial.begin(9600);

  
}
//---------------------------------------------------------------------------------------
void loop() {
  
  int key = read_key();
  
  if(key == 1) {
    current_sensor += 1;
    if(current_sensor > 3) current_sensor = 0;
    rms = temps[current_sensor];
    updateFlag = 1;  
  }
  
  if(key == 2) {
    rms = current_sensor;
    updateFlag = 2;
    t = millis();
    Serial.println(t);
  }

  if((millis() - t) >= t_delay) {
    updateFlag = 1;
    rms = temps[current_sensor];
    t = 0;
  }




    
  if((++cntr % 10000) == 0) {
    if(rms > 9999){
      rms = 0;
      updateFlag = 1;
    } 
  }

  
  if(updateFlag) // Обновим цифры, выводимые на индикатор
    UpdateDigits(rms);



  if(micros() >= stg) {
    stg = micros() + dt;
    ShowSegments(); // Подсвечиваем очередную группу сегментов. Время отработки функции - 28 мкс.
  }
}
//---------------------------------------------------------------------------------------
void UpdateDigits(float val) { // Подготовка данных (цифр) для визуализации
  if(updateFlag == 1){
    if(val >= 1000) val = 999.9;
    if(val < 0) val = 0;
    
    val = round(val * 10);
    
    d[3] = floor(val / 1000);
    d[2] = floor((val - (d[3] * 1000)) / 100 );
    d[1] = floor((val - (d[3] * 1000) - (d[2] * 100)) / 10);
    d[0] = floor (val - (d[3] * 1000) - (d[2] * 100) - (d[1] * 10));
    d[1] = d[1] + 11; // добавим точку после нужного разряда
  
    // с полным отображением числа - выключение ведущих нулей не нужно 
     if(d[3] == 0) {// Не отображаем ведущие нули ежели что... ("Цифра" 10 - это "пустая" цифра, все ее сегменты потушены)
      d[3] = 10;
      if((d[2] == 0)){
        d[2] = 10;
        if((d[1] == 0)){
          d[1] = 10;
        }
      }
    }
  } else if(updateFlag == 2){
    d[3] = 21;
    d[2] = 21;
    d[1] = 21;
    d[0] = current_sensor;
  }

  updateFlag = 0;
}
//---------------------------------------------------------------------------------------
void ShowSegments() { // Подсвечивание очередной группы сегментов
  digitalWrite(segPins[seg], L_LOW); // "Роняем" потенциал анода "отработавшего" сегмента
  if(++seg < 8) mask <<= 1; // Переходим к следующему сегменту
  else { // Дошли до последнего сегмента
    seg = 0; mask = 0x01;
  }
  digitalWrite(segPins[seg], L_HIGH); // "Подбрасываем" потенциал анода нового текущего сегмента
  for(int bi = 0; bi < 4; bi++) { // Теперь пробегаем по цифрам (разрядам) индикатора
    if(digit[d[bi]] & mask) // Если данный сегмент в bi-ом разряде должен гореть,
      digitalWrite(digPins[bi], L_LOW); // то уроним потенциал катода соответствующей цифры (разряда)
    else // А если гореть не должен, то
      digitalWrite(digPins[bi], L_HIGH); // потенциал катода bi-й цифры должен быть высокий 
  }
}


// 0 - ничего не нажато, 1 - короткое, 2 - долгое нажатие
byte read_key(){
  if (digitalRead(BUTTON) == LOW) { //считаем сколько времени нажата кнопка
    if (millis() - time_press > 1) {
      button_press_time++;
      //if(button_press_time >= 300) press_time = button_press_time;
      time_press = millis();
    }
  }
  if (digitalRead(BUTTON) == HIGH) {//очищаем счетчик если кнопка отпущена
    if(DEBUG)if(button_press_time > 0) Serial.write("Release\n");
    press_time = button_press_time;
    button_press_time = 0;
  }
  if (press_time >= 10 && press_time < 300) { // короткое нажатие
    if(DEBUG)Serial.write("Short press\n"); 
    return 1;
  } else if(press_time >= 300){ // долгое нажатие
    if(DEBUG)Serial.write("Long press\n");
    if(DEBUG)Serial.println(press_time);
    return 2;
  } else {
    return 0;
  }
}

