#include <Wire.h>

//---------------------------------------------------------------------------------------
//#define COMMON_CATHODE // Если индикатор с общим анодом - необходимо закомментировать эту строку
//---------------------------------------------------------------------------------------
#ifdef COMMON_CATHODE // Для индикаторов с общим катодом
#define L_HIGH 1
#define L_LOW  0
#else                 // Для индикаторов с общим анодом
#define L_HIGH 0
#define L_LOW  1
#endif

#define BUTTON 16     // Кнопка управления
#define DEBUG 0       // признак отладки - пишем сообщения в serial
#define ADDR 0x07     // Адрес сервера на шине I2C

uint8_t index = 0;    
uint8_t buf[17];      // Буфер для передачи по i2c
uint8_t bufL = 17;    // Длина буфера

// На данный момент задействованы все пины порта PD. Если какие-нибудь пины
// необходимо использовать для других целей (например, PD0/RXD и PD1/TXD), то
// их необходимо замаскировать (например, для только что приведенного 
// примера - mask1 = 0b11111100; mask0 = 0b00000011;
uint8_t mask1 = 0b00001111; // 4 младших пина порта D имеют датчики
                            
uint8_t mask0 = 0b11110000; // mask0 = ~mask1
uint8_t bytes[8];


int button_press_time = 0;
unsigned long time_press = 0;
int press_time = 0;
int show_mode = 0; // 0 - показ температуры, 1 - показ номера датчика

static float temps[8] = {0, 0, 0, 0, 0, 0, 0, 0};      // массив температур
byte current_sensor = 0;                                // текущий датчик


// Пины, к которым подключены сегменты индикатора аноды (или катоды для индикаторов с общими анодами)
static uint8_t segPins[8] = { 11, 15, 6, 8, 9, 12, 5, 7};
// Пины, к которым подключены общие катоды (или аноды, для индикаторов с общими анодами)
static uint8_t digPins[4] = {4, 14, 13, 10};
static uint8_t digit[23] = {0b00111111, // Маски сегментов цифр 0, 1, 2 и т.д.
                            0b00000110, //1  старший бит отвечает за точку
                            0b01011011, //2
                            0b01001111, //3
                            0b01100110, //4
                            0b01101101, //5
                            0b01111101, //6
                            0b00000111, //7
                            0b01111111, //8
                            0b01101111, //9
                            0b00000000, // "Цифра" 10 это пробел
                            
                            0b10111111, // Маски сегментов цифр 0, 1, 2 и т.д. с точками //11
                            0b10000110, //1  старший бит отвечает за точку 12
                            0b11011011, //2 13
                            0b11001111, //3 14
                            0b11100110, //4 15
                            0b11101101, //5 16
                            0b11111101, //6 17 
                            0b10000111, //7 18
                            0b11111111, //8 19 
                            0b11101111, //9 20
                            0b10000000, // 21 - точка без цифры
                            0b01000000}; // 22 - минус
                            
static int d[4]; // Тысячи, сотни, десятки и единицы показываемые на разрядах 7-сегментного индикатора
static uint8_t mask = 0x01; // Маска для анализа цифр
static int seg = 0; // Номер сегмента, обрабатываемого (подствечиваемого, если нужно) в данный момент

static unsigned long dt = 3000; // Длительность (в мкс) подсветки каждой группы сегментов
static volatile unsigned long stt = 0; // Момент времени (в мкс) когда заканчивается подсветка текущей группы сегментов
static volatile uint8_t nextSegFlag = 1; // Флаг для перехода к подсветке следующей группы сегментов
unsigned long int ttime = 0; // переменная хранения времени для датчиков
byte sstep = 1;


static volatile float rms = 0; // Среднеквадратичное значение измеряемого напряжения
static uint8_t updateFlag = 1; // 1 - данные (цифры), выводимые на индикатор, нужно обновить, 2 - выводим номер датчика


// Рабочие переменные для отладки
static volatile unsigned long stg = 0;
//static volatile int cntr = 0;
//---------------------------------------------------------------------------------------
void setup() {
  Wire.begin(ADDR); // Инициализация библиотеки для шины i2c
  Wire.onReceive(receiveData);
  Wire.onRequest(sendData);

  
  for(int i = 0; i < 8; i++) { // Инициализация пинов, к которым подключены аноды индикатора (сегменты)
    pinMode(segPins[i], OUTPUT);
    digitalWrite(segPins[i], L_LOW);
  }
  for(int i = 0; i < 4; i++) { // Инициализация пинов, к которым подключены катоды индикатора (цифры)
    pinMode(digPins[i], OUTPUT);
    digitalWrite(digPins[i], L_HIGH);
  }
  unsigned long st = millis() + 3000;
  d[0] = 8; d[1] = 8; d[2] = 8; d[3] = 8;// Для демонстрации работоспособности индикатора покажем все сегменты разом
  while(millis() < st) {
      ShowSegments();
  }
  d[0] = 10; d[1] = 10; d[2] = 10; d[3] = 10; // Очистим индикатор
  ShowSegments();

  stg = micros() + dt;
  rms = temps[current_sensor];
  // Настраиваем пин кнопки
  pinMode(BUTTON, INPUT);
  digitalWrite(BUTTON, HIGH);
  if(DEBUG) Serial.begin(9600);
  DDRD |= mask1; // Пины шин - на вывод
  PORTD |= mask1; // Отпускаем все шины
}
//---------------------------------------------------------------------------------------
void loop() {

  // кнопка
  if (digitalRead(BUTTON) == LOW) { //считаем сколько времени нажата кнопка
    if (millis() - time_press > 1) {
      button_press_time++;
      //if(button_press_time >= 300) press_time = button_press_time;
      time_press = millis();
    }
  }
  if (digitalRead(BUTTON) == HIGH) {//очищаем счетчик если кнопка отпущена
    if(DEBUG)if(button_press_time > 0) Serial.write("Release\n");
    press_time = button_press_time;
    button_press_time = 0;
    show_mode = 0;
    rms = temps[current_sensor];
    updateFlag = 1;
  }
  
  if (press_time >= 30 && press_time < 200) { // короткое нажатие
    if(DEBUG)Serial.write("Short press\n"); 
    current_sensor += 1;
    if(current_sensor > 3) current_sensor = 0;
    rms = temps[current_sensor]; 
    show_mode = 0;
    updateFlag = 1;
  } 

 if(button_press_time >= 200){ // долгое нажатие
    if(show_mode != 1) {
      if(DEBUG)Serial.write("Long press\n");
      rms = current_sensor;
      if(DEBUG)Serial.println(rms);
      updateFlag = 2;
      show_mode = 1;
    }
  }  

  // отображение информации на индикаторе
  if((show_mode == 1) && updateFlag)
    updateFlag = 2;
  if(updateFlag) // Обновим цифры, выводимые на индикатор
    UpdateDigits(rms);
  if(micros() >= stg) {
    stg = micros() + dt;
    ShowSegments(); // Подсвечиваем очередную группу сегментов. Время отработки функции - 28 мкс.
  }

  
  if(millis() >= ttime){
    uint8_t devs = w1_reset() & mask1;
    
    if(devs && sstep == 1) {  // Датчики есть
        buf[0] = devs;        // Информация о наличии датчиков - в первом байте
        w1_write_byte(0xcc);  // Skip ROM
        w1_write_byte(0x44);   // Запускаем все датчики
        sstep = 2;
    } else if(sstep == 2){      //
      sstep = 1;  
      w1_reset();
      w1_write_byte(0xcc); // Skip ROM
      w1_write_byte(0xbe); // Будем читать температуры
      w1_read_bytes(bytes);
      
      
      for(uint8_t i = 0; i < 8; i++){
        if(DEBUG)Serial.print(bytes[i]);
        if(DEBUG)Serial.print("   ");
        buf[2 + 2*i] = bytes[i];
      }
      if(DEBUG)Serial.println("");
      w1_read_bytes(bytes);
      for(uint8_t i = 0; i < 8; i++){
        if(DEBUG)Serial.print(bytes[i]);
        if(DEBUG)Serial.print("   ");
        buf[1 + 2*i] = bytes[i];
      }
      if(DEBUG)Serial.println("");if(DEBUG)Serial.println("");
        
      //w1_reset();
      fill_temps();

    }
    if(sstep == 2) ttime = millis() + 800;
  }
  
}
//---------------------------------------------------------------------------------------
void UpdateDigits(float val) { // Подготовка данных (цифр) для визуализации
  if(val == 666) {
    d[3] = current_sensor + 1;
    d[2] = d[1] = d[0] = 22; 
  } else if(updateFlag == 1){
    if(val >= 1000) val = 999.9;
    if(val < 0) val = 0;
    
    val = round(val * 10);
    
    d[3] = floor(val / 1000);
    d[2] = floor((val - (d[3] * 1000)) / 100 );
    d[1] = floor((val - (d[3] * 1000) - (d[2] * 100)) / 10);
    d[0] = floor (val - (d[3] * 1000) - (d[2] * 100) - (d[1] * 10));
    d[1] = d[1] + 11; // добавим точку после нужного разряда
  
    // с полным отображением числа - выключение ведущих нулей не нужно 
     if(d[3] == 0) {// Не отображаем ведущие нули ежели что... ("Цифра" 10 - это "пустая" цифра, все ее сегменты потушены)
      d[3] = 10;
      if((d[2] == 0)){
        d[2] = 10;
        if((d[1] == 0)){
          d[1] = 10;
        }
      }
    }
  } else if(updateFlag == 2){
    d[3] = 21;
    d[2] = 21;
    d[1] = 21;
    d[0] = current_sensor + 1;
  }

  updateFlag = 0;
}
//---------------------------------------------------------------------------------------
void ShowSegments() { // Подсвечивание очередной группы сегментов
  digitalWrite(segPins[seg], L_LOW); // "Роняем" потенциал анода "отработавшего" сегмента
  if(++seg < 8) mask <<= 1; // Переходим к следующему сегменту
  else { // Дошли до последнего сегмента
    seg = 0; mask = 0x01;
  }
  digitalWrite(segPins[seg], L_HIGH); // "Подбрасываем" потенциал анода нового текущего сегмента
  for(int bi = 0; bi < 4; bi++) { // Теперь пробегаем по цифрам (разрядам) индикатора
    if(digit[d[bi]] & mask) // Если данный сегмент в bi-ом разряде должен гореть,
      digitalWrite(digPins[bi], L_LOW); // то уроним потенциал катода соответствующей цифры (разряда)
    else // А если гореть не должен, то
      digitalWrite(digPins[bi], L_HIGH); // потенциал катода bi-й цифры должен быть высокий 
  }
}


void fill_temps(){
  uint8_t j = 0;
  for(uint8_t i = 1; i <= 15; i = i + 2){
    //Serial.print(buf[i]);
    //Serial.print("  +  ");
    //Serial.print(buf[i+1]);
    unsigned int termcode = buf[i] << 8 |  buf[i+1];
    //Serial.print("  =  ");
    //Serial.print(termcode);
    //Serial.print("     ");
    if(termcode != 65535) temps[j++] = termcode * 0.0625;
    else temps[j++] = 666;
    //Serial.println(temps[j - 1]);
  }
}

uint8_t w1_reset() { // Сброс всех шин 1-Wire и обнаружение подключенных к ним датчиков
//  cli(); // Запретим прерывания
  DDRD |= mask1; // Пины шин - на вывод
  PORTD &= mask0; // Установим низкий уровень на всех шинах 1-Wire
  _delay_us(480);
  DDRD &= mask0; // Пины шин - на ввод
  _delay_us(60);
  uint8_t devs = ~(PIND | mask0); // Там, где есть датчик - будет 1
  _delay_us(420); //ждем оставшееся время до 480мкс
  return devs;
//  sei(); // Разрешаем прерывания
}
//-----------------------------------------------------------------------------------------------------------
void w1_write_byte(uint8_t b) { // Отсылаем один и тот же байт на все шины
  uint8_t mask = 0b00000001;
  for(uint8_t bit = 0; bit < 8; bit++) {
//    cli(); // Запретим прерывания
    DDRD |= mask1; // Пины шин - на вывод
    PORTD &= mask0; // 0 на всех шинах 1-Wire
    _delay_us(1);
    if(b & mask) // Шлем единичку
      DDRD &= mask0; // "Отпустим" шины - переводем порт в режим чтения
    _delay_us(59);
    DDRD &= mask0;
    _delay_us(5);
    mask <<= 1;          
//    sei(); // Разрешаем прерывания
  }
}
//-----------------------------------------------------------------------------
void w1_read_bytes(uint8_t *b) { // Чтение байтов со всех шин 1-Wire
  uint8_t wb[8]; // Вспомогательный массив, который заносятся читаемые биты.
                 // Каждый байт wb содержит соответствующие биты всех датчиков.
                 // Например, wb[3] содержит 3-и биты всех датчиков.
//  cli(); // Запретим прерывания
  for(uint8_t bit = 0; bit < 8; bit++) {
    DDRD |= mask1; // Пины шин - на вывод
    PORTD &= mask0; // 0 на всех шинах 1-Wire
    _delay_us(1);
    DDRD &= mask0; // Пины шин - на ввод
    _delay_us(14);
    wb[bit] = PIND & mask1; // Читаем все биты с номером bit
    _delay_us(45);
  }
  _delay_us(5);
  // Преобразуем полученную информацию (массив wb) к нормальной форме, чтобы
  // каждый байт содержал все биты, полученные от одного датчика. По сути, 
  // нам нужно просто транспонировать матрицу 8x8, представленную массивом wb.
  uint8_t bmask = 0b00000001;
  for(uint8_t bit = 0; bit < 8; bit++) {
    uint8_t nmask = 0b00000001;
    for(uint8_t sn = 0; sn < 8; sn++) {
      if(buf[0] & nmask) { // Датчик на этой шине есть 
        if(wb[bit] & nmask) b[sn] |= bmask; // Установим бит
        else b[sn] &= ~bmask; // Сбросим бит
      } else b[sn] |= bmask; // Если на шине нет датчика - будут все 1
      nmask <<= 1;
    }
    bmask <<= 1;
  }
//  sei(); // Разрешаем прерывания
}

//---------------------------------------------------------------------------------------
void receiveData(int byteCount) { // callback для принимаемых по i2c байтов
  while(Wire.available()) {
    Wire.read(); // Просто затычка
  }
}
//---------------------------------------------------------------------------------------
void sendData() { // callback для передаваемых по i2c байтов
  Wire.write(buf, 17);
}
//---------------------------------------------------------------------------------------




